@@ -10,14 +10,15 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-
 import uuid
 import warnings
+from abc import abstractmethod
 from typing import Any, Callable, Dict, List, Union
 
-from braket.device_schema.device_action_properties import DeviceActionType
-from braket.device_schema.simulators import GateModelSimulatorDeviceCapabilities
-from braket.ir.jaqcd import Program
+from braket.device_schema import DeviceActionType, DeviceCapabilities
+from braket.ir.jaqcd import Program as JaqcdProgram
+from braket.ir.jaqcd.program_v1 import Results
+from braket.ir.openqasm import Program as OpenQASMProgram
 from braket.task_result import (
     AdditionalMetadata,
     GateModelTaskResult,
@@ -26,6 +27,8 @@ from braket.task_result import (
 )
 
 from braket.default_simulator.observables import Hermitian, TensorProduct
+from braket.default_simulator.openqasm.circuit import Circuit
+from braket.default_simulator.openqasm.interpreter import Interpreter
 from braket.default_simulator.operation import Observable, Operation
 from braket.default_simulator.operation_helpers import from_braket_instruction
 from braket.default_simulator.result_types import (
@@ -56,25 +59,22 @@ _NOISE_INSTRUCTIONS = frozenset(
 
 class BaseLocalSimulator(BraketSimulator):
     def run(
-        self,
-        circuit_ir: Program,
-        qubit_count: int,
-        shots: int = 0,
-        *,
-        batch_size: int = 1,
+        self, circuit_ir: Union[OpenQASMProgram, JaqcdProgram], *args, **kwargs
     ) -> GateModelTaskResult:
-        """Executes the circuit specified by the supplied `circuit_ir` on the simulator.
+        """
+        Simulate a circuit using either OpenQASM or Jaqcd.
 
         Args:
-            circuit_ir (Program): ir representation of a braket circuit specifying the
-                instructions to execute.
-            qubit_count (int): The number of qubits to simulate.
-            shots (int): The number of times to run the circuit.
-            batch_size (int): The size of the circuit partitions to contract,
+            circuit_ir (Union[OpenQASMProgram, JaqcdProgram]): Circuit specification.
+            qubit_count (int, jaqcd-only): Number of qubits.
+            shots (int, optional): The number of shots to simulate. Default is 0, which
+                performs a full analytical simulation.
+            batch_size (int, optional): The size of the circuit partitions to contract,
                 if applying multiple gates at a time is desired; see `StateVectorSimulation`.
                 Must be a positive integer.
                 Defaults to 1, which means gates are applied one at a time without any
                 optimized contraction.
+
         Returns:
             GateModelTaskResult: object that represents the result
 
@@ -83,51 +83,34 @@ class BaseLocalSimulator(BraketSimulator):
                 as a result type when shots=0. Or, if StateVector and Amplitude result types
                 are requested when shots>0.
         """
-        self._validate_ir_results_compatibility(circuit_ir)
-        self._validate_ir_instructions_compatibility(circuit_ir)
-        BaseLocalSimulator._validate_shots_and_ir_results(shots, circuit_ir, qubit_count)
+        if isinstance(circuit_ir, OpenQASMProgram):
+            return self.run_openqasm(circuit_ir, *args, **kwargs)
+        return self.run_jaqcd(circuit_ir, *args, **kwargs)
 
-        operations = [
-            from_braket_instruction(instruction) for instruction in circuit_ir.instructions
-        ]
+    @property
+    @abstractmethod
+    def properties(self) -> DeviceCapabilities:
+        """simulator properties"""
 
-        if shots > 0 and circuit_ir.basis_rotation_instructions:
-            for instruction in circuit_ir.basis_rotation_instructions:
-                operations.append(from_braket_instruction(instruction))
+    @abstractmethod
+    def initialize_simulation(self, **kwargs) -> Simulation:
+        """Initializes simulation with keyword arguments"""
 
-        BaseLocalSimulator._validate_operation_qubits(operations)
+    def _validate_ir_results_compatibility(
+        self, results: List[Results], device_action_type
+    ) -> None:
+        """
+        Validate that requested result types are valid for the simulator.
 
-        simulation = self.initialize_simulation(
-            qubit_count=qubit_count, shots=shots, batch_size=batch_size
-        )
-        simulation.evolve(operations)
+        Args:
+            results (List[Results]): Requested result types.
 
-        results = []
-
-        if not shots and circuit_ir.results:
-            result_types = BaseLocalSimulator._translate_result_types(circuit_ir)
-            BaseLocalSimulator._validate_result_types_qubits_exist(
-                [
-                    result_type
-                    for result_type in result_types
-                    if isinstance(result_type, TargetedResultType)
-                ],
-                qubit_count,
-            )
-            results = BaseLocalSimulator._generate_results(
-                circuit_ir,
-                result_types,
-                simulation,
-            )
-
-        return self._create_results_obj(results, circuit_ir, simulation)
-
-    def _validate_ir_results_compatibility(self, circuit_ir):
-        if circuit_ir.results:
-            circuit_result_types_name = [result.__class__.__name__ for result in circuit_ir.results]
-            supported_result_types = self.properties.action[
-                DeviceActionType.JAQCD
-            ].supportedResultTypes
+        Raises:
+            TypeError: If any the specified result types are not supported
+        """
+        if results:
+            circuit_result_types_name = [result.__class__.__name__ for result in results]
+            supported_result_types = self.properties.action[device_action_type].supportedResultTypes
             supported_result_types_name = [result.name for result in supported_result_types]
             for name in circuit_result_types_name:
                 if name not in supported_result_types_name:
@@ -135,50 +118,52 @@ class BaseLocalSimulator(BraketSimulator):
                         f"result type {name} is not supported by {self.__class__.__name__}"
                     )
 
-    def _validate_ir_instructions_compatibility(self, circuit_ir):
-        circuit_instruction_names = [
-            instr.__class__.__name__.lower().replace("_", "") for instr in circuit_ir.instructions
-        ]
-        supported_instructions = frozenset(
-            op.lower().replace("_", "")
-            for op in self.properties.action[DeviceActionType.JAQCD].supportedOperations
-        )
-        no_noise = True
-        for name in circuit_instruction_names:
-            if name in _NOISE_INSTRUCTIONS:
-                no_noise = False
-                if name not in supported_instructions:
-                    raise TypeError(
-                        'Noise instructions are not supported by the state vector simulator (by default). \
-You need to use the density matrix simulator: LocalSimulator("braket_dm").'
-                    )
-        if no_noise and _NOISE_INSTRUCTIONS.intersection(supported_instructions):
-            warnings.warn(
-                'You are running a noise-free circuit on the density matrix simulator. \
-Consider running this circuit on the state vector simulator: LocalSimulator("default") \
-for a better user experience.'
-            )
-
     @staticmethod
-    def _validate_shots_and_ir_results(shots: int, circuit_ir: Program, qubit_count: int) -> None:
+    def _validate_shots_and_ir_results(
+        shots: int,
+        results: List[Results],
+        qubit_count: int,
+    ) -> None:
+        """
+        Validated that requested result types are valid for given shots and qubit count.
+
+        Args:
+            shots (int): Shots for the simulation.
+            results (List[Results]): Specified result types.
+            qubit_count (int): Number of qubits for the simulation.
+
+        Raises:
+            ValueError: If any of the requested result types are incompatible with the
+                qubit count or number of shots.
+        """
         if not shots:
-            if not circuit_ir.results:
+            if not results:
                 raise ValueError("Result types must be specified in the IR when shots=0")
-            for rt in circuit_ir.results:
+            for rt in results:
                 if rt.type in ["sample"]:
                     raise ValueError("sample can only be specified when shots>0")
                 if rt.type == "amplitude":
                     BaseLocalSimulator._validate_amplitude_states(rt.states, qubit_count)
-        elif shots and circuit_ir.results:
-            for rt in circuit_ir.results:
+        elif shots and results:
+            for rt in results:
                 if rt.type in ["statevector", "amplitude", "densitymatrix"]:
                     raise ValueError(
-                        "statevector, amplitude and densitymatrix result"
+                        "statevector, amplitude and densitymatrix result "
                         "types not available when shots>0"
                     )
 
     @staticmethod
-    def _validate_amplitude_states(states: List[str], qubit_count: int):
+    def _validate_amplitude_states(states: List[str], qubit_count: int) -> None:
+        """
+        Validate states in an amplitude result type are valid.
+
+        Args:
+            states (List[str]): List of binary strings representing quantum states.
+            qubit_count (int): Number of qubits for the simulation.
+
+        Raises:
+            ValueError: If any of the states is not the correct size for the number of qubits.
+        """
         for state in states:
             if len(state) != qubit_count:
                 raise ValueError(
@@ -186,6 +171,44 @@ for a better user experience.'
                     f" must be equivalent to number of qubits {qubit_count} in circuit"
                 )
 
+    @staticmethod
+    def _translate_result_types(results: List[Results]) -> List[ResultType]:
+        return [from_braket_result_type(result) for result in results]
+
+    @staticmethod
+    def _generate_results(
+        results: List[Results],
+        result_types: List[ResultType],
+        simulation: Simulation,
+    ) -> List[ResultTypeValue]:
+        return [
+            ResultTypeValue.construct(
+                type=results[index],
+                value=result_types[index].calculate(simulation),
+            )
+            for index in range(len(results))
+        ]
+
+    def _create_results_obj(
+        self,
+        results: List[Dict[str, Any]],
+        openqasm_ir: OpenQASMProgram,
+        simulation: Simulation,
+    ) -> GateModelTaskResult:
+        return GateModelTaskResult.construct(
+            taskMetadata=TaskMetadata(
+                id=str(uuid.uuid4()),
+                shots=simulation.shots,
+                deviceId=self.DEVICE_ID,
+            ),
+            additionalMetadata=AdditionalMetadata(
+                action=openqasm_ir,
+            ),
+            resultTypes=results,
+            measurements=self._formatted_measurements(simulation),
+            measuredQubits=self._get_measured_qubits(simulation.qubit_count),
+        )
+
     @staticmethod
     def _validate_operation_qubits(operations: List[Operation]) -> None:
         qubits_referenced = {target for operation in operations for target in operation.targets}
@@ -195,18 +218,10 @@ for a better user experience.'
                 "qubit indices in a circuit must be contiguous."
             )
 
-    @staticmethod
-    def _get_measured_qubits(qubit_count: int) -> List[int]:
-        return list(range(qubit_count))
-
-    @staticmethod
-    def _translate_result_types(circuit_ir: Program) -> List[ResultType]:
-        return [from_braket_result_type(result) for result in circuit_ir.results]
-
     @staticmethod
     def _validate_result_types_qubits_exist(
         targeted_result_types: List[TargetedResultType], qubit_count: int
-    ):
+    ) -> None:
         for result_type in targeted_result_types:
             targets = result_type.targets
             if targets and max(targets) >= qubit_count:
@@ -215,6 +230,48 @@ for a better user experience.'
                     f" references invalid qubits {targets}"
                 )
 
+    def _validate_ir_instructions_compatibility(
+        self,
+        circuit_ir: Union[JaqcdProgram, Circuit],
+        device_action_type: DeviceActionType,
+    ) -> None:
+        """
+        Validate that requested IR instructions are valid for the simulator.
+
+        Args:
+            circuit_ir (Union[JaqcdProgram, Circuit]): IR for the simulator.
+
+        Raises:
+            TypeError: If any the specified result types are not supported
+        """
+        circuit_instruction_names = [
+            instr.__class__.__name__.lower().replace("_", "") for instr in circuit_ir.instructions
+        ]
+        supported_instructions = frozenset(
+            op.lower().replace("_", "")
+            for op in self.properties.action[device_action_type].supportedOperations
+        )
+        no_noise = True
+        for name in circuit_instruction_names:
+            if name in _NOISE_INSTRUCTIONS:
+                no_noise = False
+                if name not in supported_instructions:
+                    raise TypeError(
+                        "Noise instructions are not supported by the state vector simulator "
+                        "(by default). You need to use the density matrix simulator: "
+                        'LocalSimulator("braket_dm").'
+                    )
+        if no_noise and _NOISE_INSTRUCTIONS.intersection(supported_instructions):
+            warnings.warn(
+                "You are running a noise-free circuit on the density matrix simulator. "
+                "Consider running this circuit on the state vector simulator: "
+                'LocalSimulator("default") for a better user experience.'
+            )
+
+    @staticmethod
+    def _get_measured_qubits(qubit_count: int) -> List[int]:
+        return list(range(qubit_count))
+
     @staticmethod
     def _tensor_product_index_dict(
         observable: TensorProduct, callable: Callable[[Observable], Any]
@@ -245,20 +302,6 @@ for a better user experience.'
         else:
             return str(observable.__class__.__name__)
 
-    @staticmethod
-    def _generate_results(
-        circuit_ir: Program,
-        result_types: List[ResultType],
-        simulation,
-    ) -> List[ResultTypeValue]:
-        return [
-            ResultTypeValue.construct(
-                type=circuit_ir.results[index],
-                value=result_types[index].calculate(simulation),
-            )
-            for index in range(len(circuit_ir.results))
-        ]
-
     @staticmethod
     def _formatted_measurements(simulation: Simulation) -> List[List[str]]:
         """Retrieves formatted measurements obtained from the specified simulation.
@@ -275,37 +318,150 @@ for a better user experience.'
             for sample in simulation.retrieve_samples()
         ]
 
-    def _create_results_obj(
+    def run_openqasm(
         self,
-        results: List[Dict[str, Any]],
-        circuit_ir: Program,
-        simulation: Simulation,
+        openqasm_ir: OpenQASMProgram,
+        shots: int = 0,
+        *,
+        batch_size: int = 1,
     ) -> GateModelTaskResult:
-        result_dict = {
-            "taskMetadata": TaskMetadata(
-                id=str(uuid.uuid4()), shots=simulation.shots, deviceId=self.DEVICE_ID
-            ),
-            "additionalMetadata": AdditionalMetadata(action=circuit_ir),
-        }
-        if results:
-            result_dict["resultTypes"] = results
-        if simulation.shots:
-            result_dict["measurements"] = BaseLocalSimulator._formatted_measurements(simulation)
-            result_dict["measuredQubits"] = BaseLocalSimulator._get_measured_qubits(
-                simulation.qubit_count
+        """Executes the circuit specified by the supplied `circuit_ir` on the simulator.
+
+        Args:
+            openqasm_ir (Program): ir representation of a braket circuit specifying the
+                instructions to execute.
+            shots (int): The number of times to run the circuit.
+            batch_size (int): The size of the circuit partitions to contract,
+                if applying multiple gates at a time is desired; see `StateVectorSimulation`.
+                Must be a positive integer.
+                Defaults to 1, which means gates are applied one at a time without any
+                optimized contraction.
+        Returns:
+            GateModelTaskResult: object that represents the result
+
+        Raises:
+            ValueError: If result types are not specified in the IR or sample is specified
+                as a result type when shots=0. Or, if StateVector and Amplitude result types
+                are requested when shots>0.
+        """
+        is_file = openqasm_ir.source.endswith(".qasm")
+        interpreter = Interpreter()
+        circuit = interpreter.build_circuit(
+            source=openqasm_ir.source,
+            inputs=openqasm_ir.inputs,
+            is_file=is_file,
+        )
+        qubit_count = circuit.num_qubits
+
+        self._validate_ir_results_compatibility(
+            circuit.results,
+            device_action_type=DeviceActionType.OPENQASM,
+        )
+        self._validate_ir_instructions_compatibility(
+            circuit,
+            device_action_type=DeviceActionType.OPENQASM,
+        )
+        BaseLocalSimulator._validate_shots_and_ir_results(shots, circuit.results, qubit_count)
+
+        operations = circuit.instructions
+        BaseLocalSimulator._validate_operation_qubits(operations)
+
+        results = circuit.results
+
+        simulation = self.initialize_simulation(
+            qubit_count=qubit_count, shots=shots, batch_size=batch_size
+        )
+        simulation.evolve(operations)
+
+        if not shots:
+            result_types = BaseLocalSimulator._translate_result_types(results)
+            BaseLocalSimulator._validate_result_types_qubits_exist(
+                [
+                    result_type
+                    for result_type in result_types
+                    if isinstance(result_type, TargetedResultType)
+                ],
+                qubit_count,
+            )
+            results = BaseLocalSimulator._generate_results(
+                circuit.results,
+                result_types,
+                simulation,
+            )
+        else:
+            simulation.evolve(circuit.basis_rotation_instructions)
+
+        return self._create_results_obj(results, openqasm_ir, simulation)
+
+    def run_jaqcd(
+        self,
+        circuit_ir: JaqcdProgram,
+        qubit_count: int,
+        shots: int = 0,
+        *,
+        batch_size: int = 1,
+    ) -> GateModelTaskResult:
+        """Executes the circuit specified by the supplied `circuit_ir` on the simulator.
+
+        Args:
+            circuit_ir (Program): ir representation of a braket circuit specifying the
+                instructions to execute.
+            qubit_count (int): The number of qubits to simulate.
+            shots (int): The number of times to run the circuit.
+            batch_size (int): The size of the circuit partitions to contract,
+                if applying multiple gates at a time is desired; see `StateVectorSimulation`.
+                Must be a positive integer.
+                Defaults to 1, which means gates are applied one at a time without any
+                optimized contraction.
+        Returns:
+            GateModelTaskResult: object that represents the result
+
+        Raises:
+            ValueError: If result types are not specified in the IR or sample is specified
+                as a result type when shots=0. Or, if StateVector and Amplitude result types
+                are requested when shots>0.
+        """
+        self._validate_ir_results_compatibility(
+            circuit_ir.results,
+            device_action_type=DeviceActionType.JAQCD,
+        )
+        self._validate_ir_instructions_compatibility(
+            circuit_ir,
+            device_action_type=DeviceActionType.JAQCD,
+        )
+        BaseLocalSimulator._validate_shots_and_ir_results(shots, circuit_ir.results, qubit_count)
+
+        operations = [
+            from_braket_instruction(instruction) for instruction in circuit_ir.instructions
+        ]
+
+        if shots > 0 and circuit_ir.basis_rotation_instructions:
+            for instruction in circuit_ir.basis_rotation_instructions:
+                operations.append(from_braket_instruction(instruction))
+
+        BaseLocalSimulator._validate_operation_qubits(operations)
+
+        simulation = self.initialize_simulation(
+            qubit_count=qubit_count, shots=shots, batch_size=batch_size
+        )
+        simulation.evolve(operations)
+
+        results = []
+
+        if not shots and circuit_ir.results:
+            result_types = BaseLocalSimulator._translate_result_types(circuit_ir.results)
+            BaseLocalSimulator._validate_result_types_qubits_exist(
+                [
+                    result_type
+                    for result_type in result_types
+                    if isinstance(result_type, TargetedResultType)
+                ],
+                qubit_count,
+            )
+            results = self._generate_results(
+                circuit_ir.results,
+                result_types,
+                simulation,
             )
 
-        return GateModelTaskResult.construct(**result_dict)
-
-    @property
-    def properties(self) -> GateModelSimulatorDeviceCapabilities:
-        """GateModelSimulatorDeviceCapabilities: Properties of simulator such as supported IR types,
-        quantum operations, and result types.
-        """
-        raise NotImplementedError("properties has not been implemented.")
-
-    def initialize_simulation(self, **kwargs) -> Simulation:
-        """
-        Initializes simulation with keyword arguments
-        """
-        raise NotImplementedError("initialize_simulation has not been implemented.")
+        return self._create_results_obj(results, circuit_ir, simulation)
